module BasecommSdk
  class Client
    include BasecommSdk

    DEFAULT_BASE_URL = 'https://gateway.basecommerce.com'
    DEFAULT_SANDBOX_URL = 'https://gateway.basecommercesandbox.com'

    attr_reader :gateway_password, :gateway_username, :key, :sandbox
    attr_accessor :url

    def initialize(args)
      @gateway_username = args[:gateway_username]
      @gateway_password = args[:gateway_password]
      @key = args[:key]
      @sandbox = args[:sandbox]
      @url = args.fetch(:url) do
        @sandbox ? DEFAULT_SANDBOX_URL : DEFAULT_BASE_URL
      end
    end

    def get_bank_card_transaction(transaction_id)
      response = do_request(api_endpoint(__method__), transaction_id)
      decrypted_payload = decrypt_payload_to_json(
          response.body,
          :bank_card_transaction
      )
      BankCardTransaction.from_json(decrypted_payload)
    end

    def get_bank_account_transaction(transaction_id)
      response = do_request(api_endpoint(__method__), transaction_id)
      decrypted_payload = decrypt_payload_to_json(
          response.body,
          :bank_account_transaction
      )
      BankAccountTransaction.from_json(decrypted_payload)
    end

    def add_bank_account(bank_account)
      response = do_request(
          api_endpoint(__method__),
          encrypt_payload(bank_account.to_json)
      )
      decrypted_payload = decrypt_payload_to_json(
          response.body,
          :bank_account
      )
      BankAccount.from_json(decrypted_payload)
    end

    def add_bank_card(bank_card)
      json = bank_card.to_json
      logger.debug("#{__method__}: #{json}")
      response = do_request(
          api_endpoint(__method__),
          encrypt_payload(json)
      )
      decrypted_payload = decrypt_payload_to_json(
          response.body,
          :bank_card
      )
      BankCard.from_json(decrypted_payload)
    end

    def process_bank_account_transaction(bat)
      json = bat.to_json
      logger.debug("#{__method__}: #{json}")
      response = do_request(
          api_endpoint(__method__),
          encrypt_payload(json)
      )
      decrypted_payload = decrypt_payload_to_json(
          response.body,
          :bank_account_transaction
      )
      BankAccountTransaction.from_json(decrypted_payload)
    end

    def process_bank_card_transaction(bct)
      json = bct.to_json
      logger.debug("#{__method__}: #{json}")
      response = do_request(
          api_endpoint(__method__),
          encrypt_payload(json)
      )
      decrypted_payload = decrypt_payload_to_json(
          response.body,
          :bank_card_transaction
      )
      BankCardTransaction.from_json(decrypted_payload)
    end

    def ping
      json = {'PING' => 'PING'}.to_json
      endpoint = '/pcms/?f=API_PingPong'

      begin
        response = do_request(
            endpoint,
            encrypt_payload(json)
        )
        result = decrypt_payload_to_json(
            response.body,
            :success
        )
        parsed_result = JSON.parse(result)
        parsed_result.fetch('success', 'failure') == 'success'
      rescue
        false
      end
    end

    private

    # rubocop:disable Style/SingleSpaceBeforeFirstArg
    def connection
      @connection ||= Faraday.new(url: @url) do |faraday|
        faraday.request :url_encoded # form-encode POST params
        faraday.response :logger # log requests to STDOUT
        faraday.adapter :excon
      end
    end

    # rubocop:enable Style/SingleSpaceBeforeFirstArg

    def decrypt_payload_to_json(data, root_key = nil)
      tripledes = TripleDesService.new(ENV['BASECOMM_KEY'])
      decrypted_data = tripledes.decrypt(data)

      unless root_key.nil?
        hash = JSON.parse(decrypted_data)
        # Insert the root key if it is missing
        unless hash.key?(root_key.to_s)
          decrypted_data = {root_key.to_sym => hash}.to_json
        end
      end
      logger.debug("#{__method__} : Decrypted: #{decrypted_data}")
      decrypted_data
    end

    def encrypt_payload(json)
      tripledes = TripleDesService.new(ENV['BASECOMM_KEY'])
      tripledes.encrypt(json)
    end

    def api_version
      "V#{ENV['BASECOMM_API_VERSION']}"
    end

    # Pass a method name '__method__' or the actual endpoint as
    # 'API_myEndPointV4'
    def api_endpoint(method)
      if method.is_a?(Symbol)
        endpoint = method.to_s.camelize(:lower)
        "/pcms/?f=API_#{endpoint}#{api_version}"
      else
        "/pcms/?f=#{endpoint}"
      end
    end

    def do_request(endpoint, payload)
      connection.post(endpoint) do |req|
        req.headers['Content-Type'] = 'application/json'
        req.headers['Accept'] = '*/*'
        req.body = {
            gateway_username: gateway_username,
            gateway_password: gateway_password,
            payload: payload
        }.to_json
      end
    end
  end
end
